<?php
namespace Controllers;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Service\CRUD;
use Zeedhi\Framework\HTTP\Request;

class LogController {
    protected $dataSourceManager;
    protected $dataSourceName = 'usr_log_webservice';
    protected $newCodeMasterKey = 'USR_LOG_WEBSERVICE';
    protected $newCodeDrillKey = 'USR_LOG_WEBSERVICE_DRILL';
    protected $httpRequest;
    protected $method;
    protected $route;
    
    public function __construct($environment, $dataSourceManager, $newCode, $crudService) {
        $this->dataSourceManager = $dataSourceManager;
        $this->filialLogada = $environment->getCdFilial();
        $this->newCode = $newCode;
        $this->nrorg = $environment->getNrorg();
        $this->crudService = $crudService;
        //request variables
        $this->getHttpRequest();
        $this->method = Request::getMethod();
        $unprotServer = $this->accessProtected($this->httpRequest, 'server');
        $unprotParams = $this->accessProtected($unprotServer, 'parameters');
        $this->route = $unprotParams['PATH_INFO'];
    }
    
    public static function saveLog(){
        $this->saveMasterLog();
    }
    
    public function saveMasterLog($message = null, $arrToSave = null, $drillName = null, $autoSaveDrill = true){
        $newCode = $this->newCode->getCodeByTableName('USR_LOG_WEBSERVICE', $this->nrorg);
        if($arrToSave === NULL)
            $arrToSave = $this->getXMLLOG();
        $this->changeToValidDate($arrToSave, 'CUSTOMDATE1');
        $json = json_encode($arrToSave);
        $arrToSave['SEQLOG'] = $newCode;
        $arrToSave['XMLLOG']   = $json;
        $arrToSave['DSMETODO'] = $this->method;
        $arrToSave['IDACAO']   = $this->route;
        $arrToSave['OBSLOG'] = $message;
        $arrToSave['__is_new'] = true;
        
        $dataSetToSave = new DataSet("usr_log_webservice", [$arrToSave]);
        $newRequest = new DTO\Request\DataSet($dataSetToSave, '', '', '');
        $dataSetToSave = $this->crudService->getRowsToSave($newRequest, "usr_log_webservice");
        $dataSetToSave = new DataSet("usr_log_webservice", $dataSetToSave);
        $this->dataSourceManager->persist($dataSetToSave);
        
        if($autoSaveDrill){
            if($drillName === NULL){
                $arrayDrillNames = ['ITEN', 'ITENS', 'PRODUTO', 'PRODUTOS', 'PRODUCT', 'PRODUCTS', 'Produtos', 'produtos', 'produto'];
                foreach($arrayDrillNames as $detalhe){
                    if(isset($arrToSave[$detalhe])){
                        $this->saveDrillLog($message, $arrToSave[$detalhe], $newCode);
                    }
                }
            }else{
                $this->saveDrillLog($message, $arrToSave[$drillName], $newCode);
            }
        }
        return $newCode;
    }
    
    public function saveDrillLog($message = null, $arrDrill, $seqlog = 1){
        if(!is_array($arrDrill))
            $arrDrill = [$arrDrill];
            
        foreach($arrDrill as &$detalhe){
            $json = json_encode($detalhe);
            $detalhe['XMLLOG'] = $json;
            $newCode = $this->newCode->getCodeByTableName('USR_LOG_WEBSERVICE_DRILL', $this->nrorg);
            $detalhe['SEQLOG'] = $seqlog;
            $detalhe['SEQLOGDRILL'] = $newCode;
            $detalhe['OBSLOG'] = $message;
            $detalhe['DSMETODO'] = $this->method;
            $detalhe['IDACAO']   = $this->route;
            $detalhe['__is_new'] = true;
        }
        
        $dataSetToSave = new DataSet("usr_log_webservice_drill", $arrDrill);
        $newRequest = new DTO\Request\DataSet($dataSetToSave, '', '', '');
        $dataSetToSave = $this->crudService->getRowsToSave($newRequest, "usr_log_webservice_drill");
        $dataSetToSave = new DataSet("usr_log_webservice_drill", $dataSetToSave);
        $this->dataSourceManager->persist($dataSetToSave);
    }
    
    public function getXMLLOG() {
        $params = $this->httpRequest->getRequestParameters();
        $unprotParams = $this->accessProtected($params, 'parameters');
        return $unprotParams;
    } 
    
    public function getHttpRequest() {
        if ($this->httpRequest === null) {
            $this->httpRequest = Request::initFromGlobals();
        }
        return $this->httpRequest; 
    }
    
    function accessProtected($obj, $prop) {
      $reflection = new \ReflectionClass($obj);
      $property = $reflection->getProperty($prop);
      $property->setAccessible(true);
      return $property->getValue($obj);
    }
    
    //Trata o campo de data $fieldName para ser no formato d/m/Y (ptbr)
    function changeToValidDate(&$arrToSave, $fieldName){
        $dateString = "";
        if(isset($arrToSave[$fieldName])){
            $dateString = $arrToSave[$fieldName];
        }
        if(!empty($dateString)){
            
            $dateTime = \DateTime::createFromFormat('d/m/Y', $dateString);
            //Se data informada já está no formato aceito, não faz nenhuma ação
            if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
                return;
            } else {
                $dateTime = \DateTime::createFromFormat('Ymd', $dateString);
                //Se data informada no formado Ymd, muda o campo para d/m/Y
                if ($dateTime !== false && !array_sum($dateTime::getLastErrors())) {
                    $arrToSave[$fieldName] = $dateTime->format('d/m/Y');
                }
                //Se em qualquer outro formato, torna o campo NULL
                else{
                    $arrToSave[$fieldName] = null;
                }
            }
        }
    }
    //Salva o status e resposta da requisição se desejado
    public function saveStatusResponse($seqlog, $idstatus, $dsresposta){
        if(!$seqlog) return;
        $arrToSave = [];
        $arrToSave['SEQLOG'] = $seqlog;
        $arrToSave['IDSTATUS'] = $idstatus;
        $arrToSave['DSRESPOSTA'] = $dsresposta;
        $dataSetToSave = new DataSet("usr_log_webservice", [$arrToSave]);
        $newRequest = new DTO\Request\DataSet($dataSetToSave, '', '', '');
        $dataSetToSave = $this->crudService->getRowsToSave($newRequest, "usr_log_webservice");
        $dataSetToSave = new DataSet("usr_log_webservice", $dataSetToSave);
        $this->dataSourceManager->persist($dataSetToSave);
    }
}