CREATE OR REPLACE PROCEDURE GERA_AJUSTE_FECP(
   P_NRSEQUMAPAPU    IN MAPAAPURAC.NRSEQUMAPAPU%TYPE,
   P_CDEMPRESA       IN MAPAAPURAC.CDEMPRESA%TYPE,
   P_CDINSCESTA      IN MAPAAPURAC.CDINSCESTA%TYPE)
AS
   V_IDGERAFECPDEBESP VARCHAR2(01);
   V_CDAJUSTEFECPDD   VARCHAR2(10);
   V_DSAJUSTEFECPDD   VARCHAR2(250);
   V_CDAJUSTEFECPDE   VARCHAR2(10);
   V_DSAJUSTEFECPDE   VARCHAR2(250);
   V_VRADICFECP       FLOAT;
   V_NRSEQITMAPR      NUMBER;
BEGIN
   V_VRADICFECP := 0;

   /******** Verifica a parametrizao do tratamento do FECP no mapa de apurao ********/
   BEGIN
      SELECT IDGERAFECPDEBESP, CDAJUSTEFECPDD, DSAJUSTEFECPDD, CDAJUSTEFECPDE, DSAJUSTEFECPDE
        INTO V_IDGERAFECPDEBESP, V_CDAJUSTEFECPDD, V_DSAJUSTEFECPDD, V_CDAJUSTEFECPDE, V_DSAJUSTEFECPDE
        FROM PARAMESTSPED
       WHERE CDEMPRESA    = P_CDEMPRESA
         AND CDINSCESTA   = P_CDINSCESTA;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_IDGERAFECPDEBESP := 'N';
         V_CDAJUSTEFECPDD   := NULL;
         V_DSAJUSTEFECPDD   := NULL;
         V_CDAJUSTEFECPDE   := NULL;
         V_DSAJUSTEFECPDE   := NULL;
   END;

   /******** Calcula o valor do FECP presente na apurao ********/
   BEGIN
      SELECT NVL(SUM(ROUND(I.VRADICFECP,2)),0.0) AS VRADICFECP
        INTO V_VRADICFECP
        FROM IMPOSTONF I, NOTAFISCAL N
       WHERE I.NRSEQUMAPAPU  = P_NRSEQUMAPAPU
         AND I.CDFILIAL      = N.CDFILIAL
         AND I.NRLANCTONF    = N.NRLANCTONF
         AND N.IDENTRSAIDOP  = 'S'
         AND I.CDFISCALNF    = '1';
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_VRADICFECP := 0;
   END;

   /******** Atualiza o valor do campo no Mapa de Apurao ********/
   UPDATE MAPARECUP
      SET VRFECP = NVL(V_VRADICFECP,0)
    WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU;

   /******** Deleta os itens de ajuste referente ao valor do FECP gerado pela rotina, evitando erro na reapurao de mapas ********/
   DELETE FROM ITMAPARECUP
    WHERE NRSEQUMAPAPU  = P_NRSEQUMAPAPU
      AND IDTIPOITMAPR  IN ('DD','DE')
      AND IDAJFECP      = 'S';

   /******** Verifica se o FECP deve ser gerado como Dbito Especial, necessitando dos ajustes ********/
   IF (V_IDGERAFECPDEBESP = 'S') AND (V_VRADICFECP > 0) THEN
      /******** Busca o maior sequencial de Deduo ********/
      BEGIN
         SELECT NVL(MAX(NRSEQITMAPR),0) AS NRSEQITMAPR
           INTO V_NRSEQITMAPR
           FROM ITMAPARECUP
          WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
            AND IDTIPOITMAPR = 'DD';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_NRSEQITMAPR := NULL;
      END;

      V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0)+1;

      /******** Insere o item de deduo com o valor calculado ********/
      INSERT INTO ITMAPARECUP
         (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDAJUSTEITMAPA, IDAJFECP)
      VALUES
         (P_NRSEQUMAPAPU, 'DD', LPAD(V_NRSEQITMAPR,10,'0'), V_DSAJUSTEFECPDD, V_VRADICFECP, V_CDAJUSTEFECPDD, 'S');

      /******** Busca o maior sequencial de Dbito Especial ********/
      BEGIN
         SELECT NVL(MAX(NRSEQITMAPR),0) AS NRSEQITMAPR
           INTO V_NRSEQITMAPR
           FROM ITMAPARECUP
          WHERE NRSEQUMAPAPU = P_NRSEQUMAPAPU
            AND IDTIPOITMAPR = 'DE';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_NRSEQITMAPR := NULL;
      END;

      V_NRSEQITMAPR := NVL(V_NRSEQITMAPR,0)+1;

      /******** Insere o item de dbito especial com o valor calculado ********/
      INSERT INTO ITMAPARECUP
         (NRSEQUMAPAPU, IDTIPOITMAPR, NRSEQITMAPR, DSITEMMAPR, VRITEMMAPR, CDAJUSTEITMAPA, IDAJFECP)
      VALUES
         (P_NRSEQUMAPAPU, 'DE', LPAD(V_NRSEQITMAPR,10,'0'), V_DSAJUSTEFECPDE, V_VRADICFECP, V_CDAJUSTEFECPDE, 'S');
   END IF;

END;